"""
Note: ms_graph.py is code created and written by Jie Jenn.
Link to source code: https://learndataanalysis.org/ms_graph-py-source-code/
All credit goes to Jie Jenn.
"""

import webbrowser
from datetime import datetime
import json
import os
import msal

GRAPH_API_ENDPOINT = 'https://graph.microsoft.com/v1.0'

def generate_access_token(app_id, scopes):
    # Save Session Token as a token file
    access_token_cache = msal.SerializableTokenCache()

    # read the token file
    if os.path.exists('ms_graph_api_token.json'):
        access_token_cache.deserialize(open("ms_graph_api_token.json", "r").read())
        token_detail = json.load(open('ms_graph_api_token.json',))
        token_detail_key = list(token_detail['AccessToken'].keys())[0]
        token_expiration = datetime.fromtimestamp(int(token_detail['AccessToken'][token_detail_key]['expires_on']))
        if datetime.now() > token_expiration:
            os.remove('ms_graph_api_token.json')
            access_token_cache = msal.SerializableTokenCache()

    # assign a SerializableTokenCache object to the client instance
    client = msal.PublicClientApplication(client_id=app_id, token_cache=access_token_cache)

    accounts = client.get_accounts()
    if accounts:
        # load the session
        token_response = client.acquire_token_silent(scopes, accounts[0])
    else:
        # authetnicate your accoutn as usual
        flow = client.initiate_device_flow(scopes=scopes)
        print('user_code: ' + flow['user_code'])
        webbrowser.open('https://microsoft.com/devicelogin')
        token_response = client.acquire_token_by_device_flow(flow)

    with open('ms_graph_api_token.json', 'w') as _f:
        _f.write(access_token_cache.serialize())

    return token_response

if __name__ == '__main__':
    ...



# Import the os, requests, and json modules
import os, requests, json

# Import generate_access_token, GRAPH_API_ENDPOINT from ms_graph.py
from .ms_graph import generate_access_token, GRAPH_API_ENDPOINT

class OneDriveAPI:
    """The OneDriveAPi class provides OneDrive file management capabilities"""
    
    def __init__ (self, client_ID):
        """
        __init__() initializes the OneDriveAPI class

        :param client_ID: the user's client_ID
        """
        self.client_ID = client_ID;
    
    def authenticate(self, permissions):
        """
        authenticate() authenticates the user and generates an access token

        :param permissions: list of permissions to request
        :return: a dictionary containing the authorization headers
        """
        access_token = generate_access_token(self.client_ID, permissions)
        return {
            'Authorization': 'Bearer ' + access_token['access_token']
        }

    def write_file(self, folder_id, file_path):
        """
        write_file() uploads a provided file into a specific folder

        :param folder_id: the id of the folder to wite the file into
        :param file_path: the relative path of the file to input
        """
        permissions = ['Files.ReadWrite']
        headers = self.authenticate(permissions);

        file_name = os.path.basename(file_path);
        with open(file_path, 'rb') as upload:
            file_content = upload.read()
            response = requests.put(
                GRAPH_API_ENDPOINT + f'/me/drive/items/{folder_id}:/{file_name}:/content',
                headers = headers,
                data = file_content
            )
    
    def read_file(self, file_id):
        """
        read_file() downloads a user-specified file

        :param file_id: the id of the file to download
        """
        permissions = ['Files.Read.All']
        headers = self.authenticate(permissions);
        save_location = os.getcwd()

        file_data = requests.get(
            GRAPH_API_ENDPOINT + f'/me/drive/items/{file_id}',
            headers=headers,
            params={'select': 'name'}
        )
        file_name = file_data.json().get('name')

        response = requests.get(
            GRAPH_API_ENDPOINT + f'/me/drive/items/{file_id}/content', 
            headers=headers
        )
        with open(os.path.join(save_location, file_name), 'wb') as new_file:
            new_file.write(response.content)
    
    def list_folder(self, folder_id, output_name):
        """
        list_folder() gives a list of files in the provided folder

        :param folder_id: the id of the folder to retrieve its contents
        :param output_name: the name of the JSON file to output
        """
        permissions = ['Files.Read.All']
        headers = self.authenticate(permissions);

        response = requests.get(
            GRAPH_API_ENDPOINT + f'/me/drive/items/{folder_id}/children',
            headers=headers
        )
        with open(output_name, "w") as json_file:
            json.dump(response.json(), json_file, indent=4)
    
    def search_text(self, search_text, output_name):
        """
        search_files() searches for a file containing the specified search text

        :param search_text: the text to search
        :param output_name: the name of the JSON file to output
        """
        permissions = ['Files.Read.All']
        headers = self.authenticate(permissions);

        response = requests.get(
            GRAPH_API_ENDPOINT + f"/me/drive/root/search(q='{search_text}')",
            headers=headers
        )
        with open(output_name, "w") as json_file:
            json.dump(response.json(), json_file, indent=4)

    def search_id(self, search_id, output_name):
        """
        search_id() searches for a file containing the specified id

        :param search_text: the id to search
        :param output_name: the name of the JSON file to output
        """
        permissions = ['Files.Read.All']
        headers = self.authenticate(permissions);

        response = requests.get(
            GRAPH_API_ENDPOINT + f"/me/drive/items/{search_id}",
            headers=headers
        )
        with open(output_name, "w") as json_file:
            json.dump(response.json(), json_file, indent=4)


from setuptools import setup

# Read contents of README.md file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
      name='onedriveintegrate',
      version='1.4',
      long_description=long_description,
      long_description_content_type='text/markdown',
      description='Package for OneDrive File Management',
      author='William Guo',
      license='MIT',
      packages=['onedriveintegrate'],
      zip_safe=False
)